# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .wallet import Wallet
from .._models import BaseModel

__all__ = ["WalletCreateWalletsWithRecoveryResponse"]


class WalletCreateWalletsWithRecoveryResponse(BaseModel):
    recovery_user_id: str
    """The ID of the created user."""

    wallets: List[Wallet]
    """The wallets that were created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "WalletRpcParams",
    "EthSignTransaction",
    "EthSignTransactionParams",
    "EthSignTransactionParamsTransaction",
    "EthSendTransaction",
    "EthSendTransactionParams",
    "EthSendTransactionParamsTransaction",
    "PersonalSign",
    "PersonalSignParams",
    "EthSignTypedDataV4",
    "EthSignTypedDataV4Params",
    "EthSignTypedDataV4ParamsTypedData",
    "EthSignTypedDataV4ParamsTypedDataType",
    "Secp256k1Sign",
    "Secp256k1SignParams",
    "SignTransaction",
    "SignTransactionParams",
    "SignAndSendTransaction",
    "SignAndSendTransactionParams",
    "SignMessage",
    "SignMessageParams",
]


class EthSignTransaction(TypedDict, total=False):
    method: Required[Literal["eth_signTransaction"]]

    params: Required[EthSignTransactionParams]

    address: str

    chain_type: Literal["ethereum"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


_EthSignTransactionParamsTransactionReservedKeywords = TypedDict(
    "_EthSignTransactionParamsTransactionReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class EthSignTransactionParamsTransaction(_EthSignTransactionParamsTransactionReservedKeywords, total=False):
    chain_id: Union[str, int]

    data: str

    gas_limit: Union[str, int]

    gas_price: Union[str, int]

    max_fee_per_gas: Union[str, int]

    max_priority_fee_per_gas: Union[str, int]

    nonce: Union[str, int]

    to: str

    type: Literal[0, 1, 2]

    value: Union[str, int]


class EthSignTransactionParams(TypedDict, total=False):
    transaction: Required[EthSignTransactionParamsTransaction]


class EthSendTransaction(TypedDict, total=False):
    caip2: Required[str]

    method: Required[Literal["eth_sendTransaction"]]

    params: Required[EthSendTransactionParams]

    address: str

    chain_type: Literal["ethereum"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


_EthSendTransactionParamsTransactionReservedKeywords = TypedDict(
    "_EthSendTransactionParamsTransactionReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class EthSendTransactionParamsTransaction(_EthSendTransactionParamsTransactionReservedKeywords, total=False):
    chain_id: Union[str, int]

    data: str

    gas_limit: Union[str, int]

    gas_price: Union[str, int]

    max_fee_per_gas: Union[str, int]

    max_priority_fee_per_gas: Union[str, int]

    nonce: Union[str, int]

    to: str

    type: Literal[0, 1, 2]

    value: Union[str, int]


class EthSendTransactionParams(TypedDict, total=False):
    transaction: Required[EthSendTransactionParamsTransaction]


class PersonalSign(TypedDict, total=False):
    method: Required[Literal["personal_sign"]]

    params: Required[PersonalSignParams]

    address: str

    chain_type: Literal["ethereum"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class PersonalSignParams(TypedDict, total=False):
    encoding: Required[Literal["utf-8", "hex"]]

    message: Required[str]


class EthSignTypedDataV4(TypedDict, total=False):
    method: Required[Literal["eth_signTypedData_v4"]]

    params: Required[EthSignTypedDataV4Params]

    address: str

    chain_type: Literal["ethereum"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class EthSignTypedDataV4ParamsTypedDataType(TypedDict, total=False):
    name: Required[str]

    type: Required[str]


class EthSignTypedDataV4ParamsTypedData(TypedDict, total=False):
    domain: Required[Dict[str, Optional[object]]]

    message: Required[Dict[str, Optional[object]]]

    primary_type: Required[str]

    types: Required[Dict[str, Iterable[EthSignTypedDataV4ParamsTypedDataType]]]


class EthSignTypedDataV4Params(TypedDict, total=False):
    typed_data: Required[EthSignTypedDataV4ParamsTypedData]


class Secp256k1Sign(TypedDict, total=False):
    method: Required[Literal["secp256k1_sign"]]

    params: Required[Secp256k1SignParams]

    address: str

    chain_type: Literal["ethereum"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class Secp256k1SignParams(TypedDict, total=False):
    hash: object


class SignTransaction(TypedDict, total=False):
    method: Required[Literal["signTransaction"]]

    params: Required[SignTransactionParams]

    address: str

    chain_type: Literal["solana"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class SignTransactionParams(TypedDict, total=False):
    encoding: Required[Literal["base64"]]

    transaction: Required[str]


class SignAndSendTransaction(TypedDict, total=False):
    caip2: Required[str]

    method: Required[Literal["signAndSendTransaction"]]

    params: Required[SignAndSendTransactionParams]

    address: str

    chain_type: Literal["solana"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class SignAndSendTransactionParams(TypedDict, total=False):
    encoding: Required[Literal["base64"]]

    transaction: Required[str]


class SignMessage(TypedDict, total=False):
    method: Required[Literal["signMessage"]]

    params: Required[SignMessageParams]

    address: str

    chain_type: Literal["solana"]

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class SignMessageParams(TypedDict, total=False):
    encoding: Required[Literal["base64"]]

    message: Required[str]


WalletRpcParams: TypeAlias = Union[
    EthSignTransaction,
    EthSendTransaction,
    PersonalSign,
    EthSignTypedDataV4,
    Secp256k1Sign,
    SignTransaction,
    SignAndSendTransaction,
    SignMessage,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from privy_client import PrivyAPI, AsyncPrivyAPI
from privy_client.types import Policy

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPolicies:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: PrivyAPI) -> None:
        policy = client.policies.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: PrivyAPI) -> None:
        policy = client.policies.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
            owner={"public_key": "public_key"},
            owner_id="owner_id",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: PrivyAPI) -> None:
        response = client.policies.with_raw_response.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: PrivyAPI) -> None:
        with client.policies.with_streaming_response.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: PrivyAPI) -> None:
        policy = client.policies.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: PrivyAPI) -> None:
        policy = client.policies.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                }
            ],
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: PrivyAPI) -> None:
        response = client.policies.with_raw_response.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: PrivyAPI) -> None:
        with client.policies.with_streaming_response.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `policy_id` but received ''"):
            client.policies.with_raw_response.update(
                policy_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: PrivyAPI) -> None:
        policy = client.policies.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: PrivyAPI) -> None:
        policy = client.policies.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: PrivyAPI) -> None:
        response = client.policies.with_raw_response.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: PrivyAPI) -> None:
        with client.policies.with_streaming_response.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `policy_id` but received ''"):
            client.policies.with_raw_response.delete(
                policy_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: PrivyAPI) -> None:
        policy = client.policies.get(
            "xxxxxxxxxxxxxxxxxxxxxxxx",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: PrivyAPI) -> None:
        response = client.policies.with_raw_response.get(
            "xxxxxxxxxxxxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: PrivyAPI) -> None:
        with client.policies.with_streaming_response.get(
            "xxxxxxxxxxxxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `policy_id` but received ''"):
            client.policies.with_raw_response.get(
                "",
            )


class TestAsyncPolicies:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
            owner={"public_key": "public_key"},
            owner_id="owner_id",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.policies.with_raw_response.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = await response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.policies.with_streaming_response.create(
            chain_type="ethereum",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                },
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0xfde4C96c8593536E31F229EA8f37b2ADa2699bb2",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDT contract on Base",
                },
            ],
            version="1.0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = await response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
            name="Allowlisted stablecoins",
            rules=[
                {
                    "action": "ALLOW",
                    "conditions": [
                        {
                            "field": "to",
                            "field_source": "ethereum_transaction",
                            "operator": "eq",
                            "value": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
                        }
                    ],
                    "method": "eth_sendTransaction",
                    "name": "Allowlist USDC contract on Base",
                }
            ],
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.policies.with_raw_response.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = await response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.policies.with_streaming_response.update(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = await response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `policy_id` but received ''"):
            await async_client.policies.with_raw_response.update(
                policy_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.policies.with_raw_response.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = await response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.policies.with_streaming_response.delete(
            policy_id="xxxxxxxxxxxxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = await response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `policy_id` but received ''"):
            await async_client.policies.with_raw_response.delete(
                policy_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncPrivyAPI) -> None:
        policy = await async_client.policies.get(
            "xxxxxxxxxxxxxxxxxxxxxxxx",
        )
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.policies.with_raw_response.get(
            "xxxxxxxxxxxxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        policy = await response.parse()
        assert_matches_type(Policy, policy, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.policies.with_streaming_response.get(
            "xxxxxxxxxxxxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            policy = await response.parse()
            assert_matches_type(Policy, policy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `policy_id` but received ''"):
            await async_client.policies.with_raw_response.get(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Iterable, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ...types import (
    wallet_rpc_params,
    wallet_list_params,
    wallet_create_params,
    wallet_update_params,
    wallet_authenticate_with_jwt_params,
    wallet_create_wallets_with_recovery_params,
)
from .balance import (
    BalanceResource,
    AsyncBalanceResource,
    BalanceResourceWithRawResponse,
    AsyncBalanceResourceWithRawResponse,
    BalanceResourceWithStreamingResponse,
    AsyncBalanceResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import required_args, maybe_transform, strip_not_given, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.wallet import Wallet
from ...types.wallet_rpc_response import WalletRpcResponse
from ...types.wallet_authenticate_with_jwt_response import WalletAuthenticateWithJwtResponse
from ...types.wallet_create_wallets_with_recovery_response import WalletCreateWalletsWithRecoveryResponse

__all__ = ["WalletsResource", "AsyncWalletsResource"]


class WalletsResource(SyncAPIResource):
    @cached_property
    def transactions(self) -> TransactionsResource:
        return TransactionsResource(self._client)

    @cached_property
    def balance(self) -> BalanceResource:
        return BalanceResource(self._client)

    @cached_property
    def with_raw_response(self) -> WalletsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return WalletsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WalletsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return WalletsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        chain_type: Literal["solana", "ethereum", "cosmos", "stellar", "sui"],
        additional_signers: Iterable[wallet_create_params.AdditionalSigner] | NotGiven = NOT_GIVEN,
        owner: Optional[wallet_create_params.Owner] | NotGiven = NOT_GIVEN,
        owner_id: Optional[str] | NotGiven = NOT_GIVEN,
        policy_ids: List[str] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Wallet:
        """Create a new wallet.

        Args:
          chain_type: Chain type of the wallet.

        "ethereum" supports any EVM-compatible network.

          additional_signers: Additional signers for the wallet.

          owner: The P-256 public key of the owner of the wallet. If you provide this, do not
              specify an owner_id as it will be generated automatically.

          owner_id: The key quorum ID to set as the owner of the wallet. If you provide this, do not
              specify an owner.

          policy_ids: List of policy IDs for policies that should be enforced on the wallet.
              Currently, only one policy is supported per wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._post(
            "/v1/wallets",
            body=maybe_transform(
                {
                    "chain_type": chain_type,
                    "additional_signers": additional_signers,
                    "owner": owner,
                    "owner_id": owner_id,
                    "policy_ids": policy_ids,
                },
                wallet_create_params.WalletCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Wallet,
        )

    def update(
        self,
        wallet_id: str,
        *,
        additional_signers: Iterable[wallet_update_params.AdditionalSigner] | NotGiven = NOT_GIVEN,
        owner: Optional[wallet_update_params.Owner] | NotGiven = NOT_GIVEN,
        owner_id: Optional[str] | NotGiven = NOT_GIVEN,
        policy_ids: List[str] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Wallet:
        """
        Update a wallet's policies or authorization key configuration.

        Args:
          wallet_id: ID of the wallet.

          additional_signers: Additional signers for the wallet.

          owner: The P-256 public key of the owner of the wallet. If you provide this, do not
              specify an owner_id as it will be generated automatically.

          owner_id: The key quorum ID to set as the owner of the wallet. If you provide this, do not
              specify an owner.

          policy_ids: New policy IDs to enforce on the wallet. Currently, only one policy is supported
              per wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._patch(
            f"/v1/wallets/{wallet_id}",
            body=maybe_transform(
                {
                    "additional_signers": additional_signers,
                    "owner": owner,
                    "owner_id": owner_id,
                    "policy_ids": policy_ids,
                },
                wallet_update_params.WalletUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Wallet,
        )

    def list(
        self,
        *,
        chain_type: Literal["solana", "ethereum", "cosmos", "stellar", "sui"] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        limit: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[Wallet]:
        """Get all wallets in your app.

        Args:
          chain_type: Chain type of the wallet.

        'Ethereum' supports any EVM-compatible network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/wallets",
            page=SyncCursor[Wallet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "chain_type": chain_type,
                        "cursor": cursor,
                        "limit": limit,
                    },
                    wallet_list_params.WalletListParams,
                ),
            ),
            model=Wallet,
        )

    def authenticate_with_jwt(
        self,
        *,
        encryption_type: Literal["HPKE"],
        recipient_public_key: str,
        user_jwt: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletAuthenticateWithJwtResponse:
        """
        Obtain a user session signer to enable wallet access.

        Args:
          encryption_type: The encryption type for the authentication response. Currently only supports
              HPKE.

          recipient_public_key: Base64-encoded public key of the recipient who will decrypt the session key.
              This key must be generated securely and kept confidential.

          user_jwt: The user's JWT, to be used to authenticate the user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/user_signers/authenticate",
            body=maybe_transform(
                {
                    "encryption_type": encryption_type,
                    "recipient_public_key": recipient_public_key,
                    "user_jwt": user_jwt,
                },
                wallet_authenticate_with_jwt_params.WalletAuthenticateWithJwtParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WalletAuthenticateWithJwtResponse,
        )

    def create_wallets_with_recovery(
        self,
        *,
        primary_signer: wallet_create_wallets_with_recovery_params.PrimarySigner,
        recovery_user: wallet_create_wallets_with_recovery_params.RecoveryUser,
        wallets: Iterable[wallet_create_wallets_with_recovery_params.Wallet],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletCreateWalletsWithRecoveryResponse:
        """
        Create wallets with an associated recovery user.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/wallets_with_recovery",
            body=maybe_transform(
                {
                    "primary_signer": primary_signer,
                    "recovery_user": recovery_user,
                    "wallets": wallets,
                },
                wallet_create_wallets_with_recovery_params.WalletCreateWalletsWithRecoveryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WalletCreateWalletsWithRecoveryResponse,
        )

    def get(
        self,
        wallet_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Wallet:
        """
        Get a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        return self._get(
            f"/v1/wallets/{wallet_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Wallet,
        )

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["eth_signTransaction"],
        params: wallet_rpc_params.EthSignTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        caip2: str,
        method: Literal["eth_sendTransaction"],
        params: wallet_rpc_params.EthSendTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["personal_sign"],
        params: wallet_rpc_params.PersonalSignParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["eth_signTypedData_v4"],
        params: wallet_rpc_params.EthSignTypedDataV4Params,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["secp256k1_sign"],
        params: wallet_rpc_params.Secp256k1SignParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["signTransaction"],
        params: wallet_rpc_params.SignTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        caip2: str,
        method: Literal["signAndSendTransaction"],
        params: wallet_rpc_params.SignAndSendTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["signMessage"],
        params: wallet_rpc_params.SignMessageParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["method", "params"], ["caip2", "method", "params"])
    def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["eth_signTransaction"]
        | Literal["eth_sendTransaction"]
        | Literal["personal_sign"]
        | Literal["eth_signTypedData_v4"]
        | Literal["secp256k1_sign"]
        | Literal["signTransaction"]
        | Literal["signAndSendTransaction"]
        | Literal["signMessage"],
        params: wallet_rpc_params.EthSignTransactionParams
        | wallet_rpc_params.PersonalSignParams
        | wallet_rpc_params.EthSignTypedDataV4Params
        | wallet_rpc_params.Secp256k1SignParams
        | wallet_rpc_params.SignTransactionParams
        | wallet_rpc_params.SignMessageParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        caip2: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return cast(
            WalletRpcResponse,
            self._post(
                f"/v1/wallets/{wallet_id}/rpc",
                body=maybe_transform(
                    {
                        "method": method,
                        "params": params,
                        "address": address,
                        "chain_type": chain_type,
                        "caip2": caip2,
                    },
                    wallet_rpc_params.WalletRpcParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, WalletRpcResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncWalletsResource(AsyncAPIResource):
    @cached_property
    def transactions(self) -> AsyncTransactionsResource:
        return AsyncTransactionsResource(self._client)

    @cached_property
    def balance(self) -> AsyncBalanceResource:
        return AsyncBalanceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWalletsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncWalletsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWalletsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return AsyncWalletsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        chain_type: Literal["solana", "ethereum", "cosmos", "stellar", "sui"],
        additional_signers: Iterable[wallet_create_params.AdditionalSigner] | NotGiven = NOT_GIVEN,
        owner: Optional[wallet_create_params.Owner] | NotGiven = NOT_GIVEN,
        owner_id: Optional[str] | NotGiven = NOT_GIVEN,
        policy_ids: List[str] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Wallet:
        """Create a new wallet.

        Args:
          chain_type: Chain type of the wallet.

        "ethereum" supports any EVM-compatible network.

          additional_signers: Additional signers for the wallet.

          owner: The P-256 public key of the owner of the wallet. If you provide this, do not
              specify an owner_id as it will be generated automatically.

          owner_id: The key quorum ID to set as the owner of the wallet. If you provide this, do not
              specify an owner.

          policy_ids: List of policy IDs for policies that should be enforced on the wallet.
              Currently, only one policy is supported per wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._post(
            "/v1/wallets",
            body=await async_maybe_transform(
                {
                    "chain_type": chain_type,
                    "additional_signers": additional_signers,
                    "owner": owner,
                    "owner_id": owner_id,
                    "policy_ids": policy_ids,
                },
                wallet_create_params.WalletCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Wallet,
        )

    async def update(
        self,
        wallet_id: str,
        *,
        additional_signers: Iterable[wallet_update_params.AdditionalSigner] | NotGiven = NOT_GIVEN,
        owner: Optional[wallet_update_params.Owner] | NotGiven = NOT_GIVEN,
        owner_id: Optional[str] | NotGiven = NOT_GIVEN,
        policy_ids: List[str] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Wallet:
        """
        Update a wallet's policies or authorization key configuration.

        Args:
          wallet_id: ID of the wallet.

          additional_signers: Additional signers for the wallet.

          owner: The P-256 public key of the owner of the wallet. If you provide this, do not
              specify an owner_id as it will be generated automatically.

          owner_id: The key quorum ID to set as the owner of the wallet. If you provide this, do not
              specify an owner.

          policy_ids: New policy IDs to enforce on the wallet. Currently, only one policy is supported
              per wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._patch(
            f"/v1/wallets/{wallet_id}",
            body=await async_maybe_transform(
                {
                    "additional_signers": additional_signers,
                    "owner": owner,
                    "owner_id": owner_id,
                    "policy_ids": policy_ids,
                },
                wallet_update_params.WalletUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Wallet,
        )

    def list(
        self,
        *,
        chain_type: Literal["solana", "ethereum", "cosmos", "stellar", "sui"] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        limit: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Wallet, AsyncCursor[Wallet]]:
        """Get all wallets in your app.

        Args:
          chain_type: Chain type of the wallet.

        'Ethereum' supports any EVM-compatible network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/wallets",
            page=AsyncCursor[Wallet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "chain_type": chain_type,
                        "cursor": cursor,
                        "limit": limit,
                    },
                    wallet_list_params.WalletListParams,
                ),
            ),
            model=Wallet,
        )

    async def authenticate_with_jwt(
        self,
        *,
        encryption_type: Literal["HPKE"],
        recipient_public_key: str,
        user_jwt: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletAuthenticateWithJwtResponse:
        """
        Obtain a user session signer to enable wallet access.

        Args:
          encryption_type: The encryption type for the authentication response. Currently only supports
              HPKE.

          recipient_public_key: Base64-encoded public key of the recipient who will decrypt the session key.
              This key must be generated securely and kept confidential.

          user_jwt: The user's JWT, to be used to authenticate the user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/user_signers/authenticate",
            body=await async_maybe_transform(
                {
                    "encryption_type": encryption_type,
                    "recipient_public_key": recipient_public_key,
                    "user_jwt": user_jwt,
                },
                wallet_authenticate_with_jwt_params.WalletAuthenticateWithJwtParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WalletAuthenticateWithJwtResponse,
        )

    async def create_wallets_with_recovery(
        self,
        *,
        primary_signer: wallet_create_wallets_with_recovery_params.PrimarySigner,
        recovery_user: wallet_create_wallets_with_recovery_params.RecoveryUser,
        wallets: Iterable[wallet_create_wallets_with_recovery_params.Wallet],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletCreateWalletsWithRecoveryResponse:
        """
        Create wallets with an associated recovery user.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/wallets_with_recovery",
            body=await async_maybe_transform(
                {
                    "primary_signer": primary_signer,
                    "recovery_user": recovery_user,
                    "wallets": wallets,
                },
                wallet_create_wallets_with_recovery_params.WalletCreateWalletsWithRecoveryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WalletCreateWalletsWithRecoveryResponse,
        )

    async def get(
        self,
        wallet_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Wallet:
        """
        Get a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        return await self._get(
            f"/v1/wallets/{wallet_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Wallet,
        )

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["eth_signTransaction"],
        params: wallet_rpc_params.EthSignTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        caip2: str,
        method: Literal["eth_sendTransaction"],
        params: wallet_rpc_params.EthSendTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["personal_sign"],
        params: wallet_rpc_params.PersonalSignParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["eth_signTypedData_v4"],
        params: wallet_rpc_params.EthSignTypedDataV4Params,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["secp256k1_sign"],
        params: wallet_rpc_params.Secp256k1SignParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["signTransaction"],
        params: wallet_rpc_params.SignTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        caip2: str,
        method: Literal["signAndSendTransaction"],
        params: wallet_rpc_params.SignAndSendTransactionParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["signMessage"],
        params: wallet_rpc_params.SignMessageParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        """
        Sign a message or transaction with a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["method", "params"], ["caip2", "method", "params"])
    async def rpc(
        self,
        wallet_id: str,
        *,
        method: Literal["eth_signTransaction"]
        | Literal["eth_sendTransaction"]
        | Literal["personal_sign"]
        | Literal["eth_signTypedData_v4"]
        | Literal["secp256k1_sign"]
        | Literal["signTransaction"]
        | Literal["signAndSendTransaction"]
        | Literal["signMessage"],
        params: wallet_rpc_params.EthSignTransactionParams
        | wallet_rpc_params.PersonalSignParams
        | wallet_rpc_params.EthSignTypedDataV4Params
        | wallet_rpc_params.Secp256k1SignParams
        | wallet_rpc_params.SignTransactionParams
        | wallet_rpc_params.SignMessageParams,
        address: str | NotGiven = NOT_GIVEN,
        chain_type: Literal["ethereum"] | Literal["solana"] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        caip2: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WalletRpcResponse:
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return cast(
            WalletRpcResponse,
            await self._post(
                f"/v1/wallets/{wallet_id}/rpc",
                body=await async_maybe_transform(
                    {
                        "method": method,
                        "params": params,
                        "address": address,
                        "chain_type": chain_type,
                        "caip2": caip2,
                    },
                    wallet_rpc_params.WalletRpcParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, WalletRpcResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class WalletsResourceWithRawResponse:
    def __init__(self, wallets: WalletsResource) -> None:
        self._wallets = wallets

        self.create = to_raw_response_wrapper(
            wallets.create,
        )
        self.update = to_raw_response_wrapper(
            wallets.update,
        )
        self.list = to_raw_response_wrapper(
            wallets.list,
        )
        self.authenticate_with_jwt = to_raw_response_wrapper(
            wallets.authenticate_with_jwt,
        )
        self.create_wallets_with_recovery = to_raw_response_wrapper(
            wallets.create_wallets_with_recovery,
        )
        self.get = to_raw_response_wrapper(
            wallets.get,
        )
        self.rpc = to_raw_response_wrapper(
            wallets.rpc,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithRawResponse:
        return TransactionsResourceWithRawResponse(self._wallets.transactions)

    @cached_property
    def balance(self) -> BalanceResourceWithRawResponse:
        return BalanceResourceWithRawResponse(self._wallets.balance)


class AsyncWalletsResourceWithRawResponse:
    def __init__(self, wallets: AsyncWalletsResource) -> None:
        self._wallets = wallets

        self.create = async_to_raw_response_wrapper(
            wallets.create,
        )
        self.update = async_to_raw_response_wrapper(
            wallets.update,
        )
        self.list = async_to_raw_response_wrapper(
            wallets.list,
        )
        self.authenticate_with_jwt = async_to_raw_response_wrapper(
            wallets.authenticate_with_jwt,
        )
        self.create_wallets_with_recovery = async_to_raw_response_wrapper(
            wallets.create_wallets_with_recovery,
        )
        self.get = async_to_raw_response_wrapper(
            wallets.get,
        )
        self.rpc = async_to_raw_response_wrapper(
            wallets.rpc,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithRawResponse:
        return AsyncTransactionsResourceWithRawResponse(self._wallets.transactions)

    @cached_property
    def balance(self) -> AsyncBalanceResourceWithRawResponse:
        return AsyncBalanceResourceWithRawResponse(self._wallets.balance)


class WalletsResourceWithStreamingResponse:
    def __init__(self, wallets: WalletsResource) -> None:
        self._wallets = wallets

        self.create = to_streamed_response_wrapper(
            wallets.create,
        )
        self.update = to_streamed_response_wrapper(
            wallets.update,
        )
        self.list = to_streamed_response_wrapper(
            wallets.list,
        )
        self.authenticate_with_jwt = to_streamed_response_wrapper(
            wallets.authenticate_with_jwt,
        )
        self.create_wallets_with_recovery = to_streamed_response_wrapper(
            wallets.create_wallets_with_recovery,
        )
        self.get = to_streamed_response_wrapper(
            wallets.get,
        )
        self.rpc = to_streamed_response_wrapper(
            wallets.rpc,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithStreamingResponse:
        return TransactionsResourceWithStreamingResponse(self._wallets.transactions)

    @cached_property
    def balance(self) -> BalanceResourceWithStreamingResponse:
        return BalanceResourceWithStreamingResponse(self._wallets.balance)


class AsyncWalletsResourceWithStreamingResponse:
    def __init__(self, wallets: AsyncWalletsResource) -> None:
        self._wallets = wallets

        self.create = async_to_streamed_response_wrapper(
            wallets.create,
        )
        self.update = async_to_streamed_response_wrapper(
            wallets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            wallets.list,
        )
        self.authenticate_with_jwt = async_to_streamed_response_wrapper(
            wallets.authenticate_with_jwt,
        )
        self.create_wallets_with_recovery = async_to_streamed_response_wrapper(
            wallets.create_wallets_with_recovery,
        )
        self.get = async_to_streamed_response_wrapper(
            wallets.get,
        )
        self.rpc = async_to_streamed_response_wrapper(
            wallets.rpc,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithStreamingResponse:
        return AsyncTransactionsResourceWithStreamingResponse(self._wallets.transactions)

    @cached_property
    def balance(self) -> AsyncBalanceResourceWithStreamingResponse:
        return AsyncBalanceResourceWithStreamingResponse(self._wallets.balance)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "UserCreateParams",
    "LinkedAccount",
    "LinkedAccountWallet",
    "LinkedAccountEmail",
    "LinkedAccountPhone",
    "LinkedAccountGoogle",
    "LinkedAccountTwitter",
    "LinkedAccountDiscord",
    "LinkedAccountGitHub",
    "LinkedAccountSpotify",
    "LinkedAccountInstagram",
    "LinkedAccountTiktok",
    "LinkedAccountApple",
    "LinkedAccountLinkedIn",
    "LinkedAccountFarcaster",
    "LinkedAccountTelegram",
    "LinkedAccountCustomJwt",
]


class UserCreateParams(TypedDict, total=False):
    linked_accounts: Required[Iterable[LinkedAccount]]

    create_ethereum_smart_wallet: bool
    """Create a smart wallet. Requires create_ethereum_wallet to also be true."""

    create_ethereum_wallet: bool
    """Create an Ethereum embedded wallet.

    Cannot be used with create_n_ethereum_wallets.
    """

    create_n_ethereum_wallets: float
    """Number of Ethereum embedded wallets to pregenerate.

    Cannot be used with create_ethereum_wallet.
    """

    create_solana_wallet: bool
    """Create a Solana embedded wallet."""

    custom_metadata: Dict[str, Union[str, float, bool]]
    """Custom metadata associated with the user."""


class LinkedAccountWallet(TypedDict, total=False):
    address: Required[str]

    chain_type: Required[Literal["ethereum", "solana"]]

    type: Required[Literal["wallet"]]


class LinkedAccountEmail(TypedDict, total=False):
    address: Required[str]

    type: Required[Literal["email"]]


class LinkedAccountPhone(TypedDict, total=False):
    number: Required[str]

    type: Required[Literal["phone"]]


class LinkedAccountGoogle(TypedDict, total=False):
    email: Required[str]

    name: Required[str]

    subject: Required[str]

    type: Required[Literal["google_oauth"]]


class LinkedAccountTwitter(TypedDict, total=False):
    name: Required[str]

    subject: Required[str]

    type: Required[Literal["twitter_oauth"]]

    username: Required[str]

    profile_picture_url: str


class LinkedAccountDiscord(TypedDict, total=False):
    subject: Required[str]

    type: Required[Literal["discord_oauth"]]

    username: Required[str]

    email: str


class LinkedAccountGitHub(TypedDict, total=False):
    subject: Required[str]

    type: Required[Literal["github_oauth"]]

    username: Required[str]

    email: str

    name: str


class LinkedAccountSpotify(TypedDict, total=False):
    subject: Required[str]

    type: Required[Literal["spotify_oauth"]]

    email: str

    name: str


class LinkedAccountInstagram(TypedDict, total=False):
    subject: Required[str]

    type: Required[Literal["instagram_oauth"]]

    username: Required[str]


class LinkedAccountTiktok(TypedDict, total=False):
    name: Required[Optional[str]]

    subject: Required[str]

    type: Required[Literal["tiktok_oauth"]]

    username: Required[str]


class LinkedAccountApple(TypedDict, total=False):
    subject: Required[str]

    type: Required[Literal["apple_oauth"]]

    email: str


class LinkedAccountLinkedIn(TypedDict, total=False):
    subject: Required[str]

    type: Required[Literal["linkedin_oauth"]]

    email: str

    name: str

    vanity_name: Annotated[str, PropertyInfo(alias="vanityName")]


class LinkedAccountFarcaster(TypedDict, total=False):
    fid: Required[int]

    owner_address: Required[str]

    type: Required[Literal["farcaster"]]

    bio: str

    display_name: str

    homepage_url: str

    profile_picture_url: str

    username: str


class LinkedAccountTelegram(TypedDict, total=False):
    telegram_user_id: Required[str]

    type: Required[Literal["telegram"]]

    first_name: str

    last_name: str

    photo_url: str

    username: str


class LinkedAccountCustomJwt(TypedDict, total=False):
    custom_user_id: Required[str]

    type: Required[Literal["custom_auth"]]


LinkedAccount: TypeAlias = Union[
    LinkedAccountWallet,
    LinkedAccountEmail,
    LinkedAccountPhone,
    LinkedAccountGoogle,
    LinkedAccountTwitter,
    LinkedAccountDiscord,
    LinkedAccountGitHub,
    LinkedAccountSpotify,
    LinkedAccountInstagram,
    LinkedAccountTiktok,
    LinkedAccountApple,
    LinkedAccountLinkedIn,
    LinkedAccountFarcaster,
    LinkedAccountTelegram,
    LinkedAccountCustomJwt,
]



from typing import Any, cast
from functools import partial
from urllib.parse import unquote

import pytest

from privy_client._qs import Querystring, stringify


def test_empty() -> None:
    assert stringify({}) == ""
    assert stringify({"a": {}}) == ""
    assert stringify({"a": {"b": {"c": {}}}}) == ""


def test_basic() -> None:
    assert stringify({"a": 1}) == "a=1"
    assert stringify({"a": "b"}) == "a=b"
    assert stringify({"a": True}) == "a=true"
    assert stringify({"a": False}) == "a=false"
    assert stringify({"a": 1.23456}) == "a=1.23456"
    assert stringify({"a": None}) == ""


@pytest.mark.parametrize("method", ["class", "function"])
def test_nested_dotted(method: str) -> None:
    if method == "class":
        serialise = Querystring(nested_format="dots").stringify
    else:
        serialise = partial(stringify, nested_format="dots")

    assert unquote(serialise({"a": {"b": "c"}})) == "a.b=c"
    assert unquote(serialise({"a": {"b": "c", "d": "e", "f": "g"}})) == "a.b=c&a.d=e&a.f=g"
    assert unquote(serialise({"a": {"b": {"c": {"d": "e"}}}})) == "a.b.c.d=e"
    assert unquote(serialise({"a": {"b": True}})) == "a.b=true"


def test_nested_brackets() -> None:
    assert unquote(stringify({"a": {"b": "c"}})) == "a[b]=c"
    assert unquote(stringify({"a": {"b": "c", "d": "e", "f": "g"}})) == "a[b]=c&a[d]=e&a[f]=g"
    assert unquote(stringify({"a": {"b": {"c": {"d": "e"}}}})) == "a[b][c][d]=e"
    assert unquote(stringify({"a": {"b": True}})) == "a[b]=true"


@pytest.mark.parametrize("method", ["class", "function"])
def test_array_comma(method: str) -> None:
    if method == "class":
        serialise = Querystring(array_format="comma").stringify
    else:
        serialise = partial(stringify, array_format="comma")

    assert unquote(serialise({"in": ["foo", "bar"]})) == "in=foo,bar"
    assert unquote(serialise({"a": {"b": [True, False]}})) == "a[b]=true,false"
    assert unquote(serialise({"a": {"b": [True, False, None, True]}})) == "a[b]=true,false,true"


def test_array_repeat() -> None:
    assert unquote(stringify({"in": ["foo", "bar"]})) == "in=foo&in=bar"
    assert unquote(stringify({"a": {"b": [True, False]}})) == "a[b]=true&a[b]=false"
    assert unquote(stringify({"a": {"b": [True, False, None, True]}})) == "a[b]=true&a[b]=false&a[b]=true"
    assert unquote(stringify({"in": ["foo", {"b": {"c": ["d", "e"]}}]})) == "in=foo&in[b][c]=d&in[b][c]=e"


@pytest.mark.parametrize("method", ["class", "function"])
def test_array_brackets(method: str) -> None:
    if method == "class":
        serialise = Querystring(array_format="brackets").stringify
    else:
        serialise = partial(stringify, array_format="brackets")

    assert unquote(serialise({"in": ["foo", "bar"]})) == "in[]=foo&in[]=bar"
    assert unquote(serialise({"a": {"b": [True, False]}})) == "a[b][]=true&a[b][]=false"
    assert unquote(serialise({"a": {"b": [True, False, None, True]}})) == "a[b][]=true&a[b][]=false&a[b][]=true"


def test_unknown_array_format() -> None:
    with pytest.raises(NotImplementedError, match="Unknown array_format value: foo, choose from comma, repeat"):
        stringify({"a": ["foo", "bar"]}, array_format=cast(Any, "foo"))



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import key_quorum_create_params, key_quorum_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, strip_not_given, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.key_quorum import KeyQuorum

__all__ = ["KeyQuorumsResource", "AsyncKeyQuorumsResource"]


class KeyQuorumsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> KeyQuorumsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return KeyQuorumsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KeyQuorumsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return KeyQuorumsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        public_keys: List[str],
        authorization_threshold: float | NotGiven = NOT_GIVEN,
        display_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Create a new key quorum.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/key_quorums",
            body=maybe_transform(
                {
                    "public_keys": public_keys,
                    "authorization_threshold": authorization_threshold,
                    "display_name": display_name,
                },
                key_quorum_create_params.KeyQuorumCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )

    def update(
        self,
        key_quorum_id: str,
        *,
        public_keys: List[str],
        authorization_threshold: float | NotGiven = NOT_GIVEN,
        display_name: str | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Update a key quorum by key quorum ID.

        Args:
          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_quorum_id:
            raise ValueError(f"Expected a non-empty value for `key_quorum_id` but received {key_quorum_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._patch(
            f"/v1/key_quorums/{key_quorum_id}",
            body=maybe_transform(
                {
                    "public_keys": public_keys,
                    "authorization_threshold": authorization_threshold,
                    "display_name": display_name,
                },
                key_quorum_update_params.KeyQuorumUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )

    def delete(
        self,
        key_quorum_id: str,
        *,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Delete a key quorum by key quorum ID.

        Args:
          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_quorum_id:
            raise ValueError(f"Expected a non-empty value for `key_quorum_id` but received {key_quorum_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._delete(
            f"/v1/key_quorums/{key_quorum_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )

    def get(
        self,
        key_quorum_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Get a key quorum by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_quorum_id:
            raise ValueError(f"Expected a non-empty value for `key_quorum_id` but received {key_quorum_id!r}")
        return self._get(
            f"/v1/key_quorums/{key_quorum_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )


class AsyncKeyQuorumsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncKeyQuorumsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncKeyQuorumsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKeyQuorumsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return AsyncKeyQuorumsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        public_keys: List[str],
        authorization_threshold: float | NotGiven = NOT_GIVEN,
        display_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Create a new key quorum.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/key_quorums",
            body=await async_maybe_transform(
                {
                    "public_keys": public_keys,
                    "authorization_threshold": authorization_threshold,
                    "display_name": display_name,
                },
                key_quorum_create_params.KeyQuorumCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )

    async def update(
        self,
        key_quorum_id: str,
        *,
        public_keys: List[str],
        authorization_threshold: float | NotGiven = NOT_GIVEN,
        display_name: str | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Update a key quorum by key quorum ID.

        Args:
          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_quorum_id:
            raise ValueError(f"Expected a non-empty value for `key_quorum_id` but received {key_quorum_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._patch(
            f"/v1/key_quorums/{key_quorum_id}",
            body=await async_maybe_transform(
                {
                    "public_keys": public_keys,
                    "authorization_threshold": authorization_threshold,
                    "display_name": display_name,
                },
                key_quorum_update_params.KeyQuorumUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )

    async def delete(
        self,
        key_quorum_id: str,
        *,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Delete a key quorum by key quorum ID.

        Args:
          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_quorum_id:
            raise ValueError(f"Expected a non-empty value for `key_quorum_id` but received {key_quorum_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._delete(
            f"/v1/key_quorums/{key_quorum_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )

    async def get(
        self,
        key_quorum_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KeyQuorum:
        """
        Get a key quorum by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_quorum_id:
            raise ValueError(f"Expected a non-empty value for `key_quorum_id` but received {key_quorum_id!r}")
        return await self._get(
            f"/v1/key_quorums/{key_quorum_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KeyQuorum,
        )


class KeyQuorumsResourceWithRawResponse:
    def __init__(self, key_quorums: KeyQuorumsResource) -> None:
        self._key_quorums = key_quorums

        self.create = to_raw_response_wrapper(
            key_quorums.create,
        )
        self.update = to_raw_response_wrapper(
            key_quorums.update,
        )
        self.delete = to_raw_response_wrapper(
            key_quorums.delete,
        )
        self.get = to_raw_response_wrapper(
            key_quorums.get,
        )


class AsyncKeyQuorumsResourceWithRawResponse:
    def __init__(self, key_quorums: AsyncKeyQuorumsResource) -> None:
        self._key_quorums = key_quorums

        self.create = async_to_raw_response_wrapper(
            key_quorums.create,
        )
        self.update = async_to_raw_response_wrapper(
            key_quorums.update,
        )
        self.delete = async_to_raw_response_wrapper(
            key_quorums.delete,
        )
        self.get = async_to_raw_response_wrapper(
            key_quorums.get,
        )


class KeyQuorumsResourceWithStreamingResponse:
    def __init__(self, key_quorums: KeyQuorumsResource) -> None:
        self._key_quorums = key_quorums

        self.create = to_streamed_response_wrapper(
            key_quorums.create,
        )
        self.update = to_streamed_response_wrapper(
            key_quorums.update,
        )
        self.delete = to_streamed_response_wrapper(
            key_quorums.delete,
        )
        self.get = to_streamed_response_wrapper(
            key_quorums.get,
        )


class AsyncKeyQuorumsResourceWithStreamingResponse:
    def __init__(self, key_quorums: AsyncKeyQuorumsResource) -> None:
        self._key_quorums = key_quorums

        self.create = async_to_streamed_response_wrapper(
            key_quorums.create,
        )
        self.update = async_to_streamed_response_wrapper(
            key_quorums.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            key_quorums.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            key_quorums.get,
        )
